@@ -3,11 +3,11 @@
 
 #include "lilv_test_utils.h"
 
-#include "../src/filesystem.h"
-#include "../src/lilv_internal.h"
-
 #include "lilv/lilv.h"
 #include "serd/serd.h"
+#include "zix/allocator.h"
+#include "zix/filesystem.h"
+#include "zix/path.h"
 
 #include <errno.h>
 #include <stdbool.h>
@@ -31,13 +31,13 @@ lilv_test_env_new(void)
   env->plugin2_uri = lilv_new_uri(world, "http://example.org/foobar");
 
   // Set custom LV2_PATH in build directory to only use test data
-  char*     test_path = lilv_path_canonical(LILV_TEST_DIR);
-  char*     lv2_path  = lilv_strjoin(test_path, "/lv2", NULL);
+  char*     test_path = zix_canonical_path(NULL, LILV_TEST_DIR);
+  char*     lv2_path  = zix_path_join(NULL, test_path, "lv2");
   LilvNode* path      = lilv_new_string(world, lv2_path);
   lilv_world_set_option(world, LILV_OPTION_LV2_PATH, path);
-  free(lv2_path);
-  free(test_path);
   lilv_node_free(path);
+  zix_free(NULL, lv2_path);
+  zix_free(NULL, test_path);
 
   return env;
 }
@@ -45,10 +45,10 @@ lilv_test_env_new(void)
 void
 lilv_test_env_free(LilvTestEnv* env)
 {
-  free(env->test_content_path);
-  free(env->test_manifest_path);
-  free(env->test_bundle_uri);
-  free(env->test_bundle_path);
+  zix_free(NULL, env->test_content_path);
+  zix_free(NULL, env->test_manifest_path);
+  lilv_node_free(env->test_bundle_uri);
+  zix_free(NULL, env->test_bundle_path);
   lilv_node_free(env->plugin2_uri);
   lilv_node_free(env->plugin1_uri);
   lilv_world_free(env->world);
@@ -62,16 +62,16 @@ create_bundle(LilvTestEnv* env,
               const char*  plugin)
 {
   {
-    char* const test_dir   = lilv_path_canonical(LILV_TEST_DIR);
-    char* const bundle_dir = lilv_path_join(test_dir, name);
+    char* const test_dir   = zix_canonical_path(NULL, LILV_TEST_DIR);
+    char* const bundle_dir = zix_path_join(NULL, test_dir, name);
 
-    env->test_bundle_path = lilv_path_join(bundle_dir, "");
+    env->test_bundle_path = zix_path_join(NULL, bundle_dir, "");
 
-    lilv_free(bundle_dir);
-    lilv_free(test_dir);
+    zix_free(NULL, bundle_dir);
+    zix_free(NULL, test_dir);
   }
 
-  if (lilv_create_directories(env->test_bundle_path)) {
+  if (zix_create_directories(NULL, env->test_bundle_path)) {
     fprintf(stderr,
             "Failed to create directory '%s' (%s)\n",
             env->test_bundle_path,
@@ -83,9 +83,12 @@ create_bundle(LilvTestEnv* env,
     (const uint8_t*)env->test_bundle_path, NULL, NULL, true);
 
   env->test_bundle_uri = lilv_new_uri(env->world, (const char*)s.buf);
+
   env->test_manifest_path =
-    lilv_path_join(env->test_bundle_path, "manifest.ttl");
-  env->test_content_path = lilv_path_join(env->test_bundle_path, "plugin.ttl");
+    zix_path_join(NULL, env->test_bundle_path, "manifest.ttl");
+
+  env->test_content_path =
+    zix_path_join(NULL, env->test_bundle_path, "plugin.ttl");
 
   serd_node_free(&s);
 
@@ -137,21 +140,21 @@ void
 delete_bundle(LilvTestEnv* env)
 {
   if (env->test_content_path) {
-    lilv_remove(env->test_content_path);
+    zix_remove(env->test_content_path);
   }
 
   if (env->test_manifest_path) {
-    lilv_remove(env->test_manifest_path);
+    zix_remove(env->test_manifest_path);
   }
 
   if (env->test_bundle_path) {
     remove(env->test_bundle_path);
   }
 
-  free(env->test_content_path);
-  free(env->test_manifest_path);
-  free(env->test_bundle_uri);
-  free(env->test_bundle_path);
+  zix_free(NULL, env->test_content_path);
+  zix_free(NULL, env->test_manifest_path);
+  lilv_node_free(env->test_bundle_uri);
+  zix_free(NULL, env->test_bundle_path);
 
   env->test_content_path  = NULL;
   env->test_manifest_path = NULL;
@@ -173,3 +176,16 @@ set_env(const char* name, const char* value)
   setenv(name, value, 1);
 #endif
 }
+
+char*
+lilv_create_temporary_directory(const char* pattern)
+{
+  char* const tmpdir       = zix_temp_directory_path(NULL);
+  char* const path_pattern = zix_path_join(NULL, tmpdir, pattern);
+  char* const result       = zix_create_temporary_directory(NULL, path_pattern);
+
+  zix_free(NULL, path_pattern);
+  zix_free(NULL, tmpdir);
+
+  return result;
+}
