@@ -7,7 +7,7 @@
 #include "lv2/urid/urid.h"
 
 #ifdef _WIN32
-#  define _WIN32_LEAN_AND_MEAN
+#  define WIN32_LEAN_AND_MEAN
 #  include <windows.h>
 #endif
 
@@ -305,7 +305,7 @@ save(LV2_Handle                instance,
     char* apath  = map_path->abstract_path(map_path->handle, tmp_file_path);
     char* apath2 = map_path->abstract_path(map_path->handle, tmp_file_path);
     free(tmp_file_path);
-    if (strcmp(apath, apath2)) {
+    if (!!strcmp(apath, apath2)) {
       fprintf(stderr, "error: Path %s != %s\n", apath, apath2);
     }
 
@@ -407,7 +407,7 @@ restore(LV2_Handle                  instance,
     char   str[8];
     size_t n_read = fread(str, 1, sizeof(str), f);
     fclose(f);
-    if (strncmp(str, "Hello\n", n_read)) {
+    if (!!strncmp(str, "Hello\n", n_read)) {
       fprintf(
         stderr, "error: Restored bad file contents `%s' != `Hello'\n", str);
     }
@@ -458,10 +458,5 @@ LV2_SYMBOL_EXPORT
 const LV2_Descriptor*
 lv2_descriptor(uint32_t index)
 {
-  switch (index) {
-  case 0:
-    return &descriptor;
-  default:
-    return NULL;
-  }
+  return index ? NULL : &descriptor;
 }
